package com.ruoyi.demo.config;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ruoyi.common.utils.redis.RedisUtils;
import com.ruoyi.demo.constant.RedisInitContant;
import com.ruoyi.demo.entity.AddrCategory;
import com.ruoyi.demo.entity.ManageType;
import com.ruoyi.demo.entity.TypeBy;
import com.ruoyi.demo.mapper.AddrCategoryDao;
import com.ruoyi.demo.mapper.TypeByDao;
import com.ruoyi.demo.service.ManageTypeService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;

import javax.annotation.PostConstruct;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.concurrent.ThreadPoolExecutor;

@Slf4j
@Configuration
public class BenyunCoreInit {
    @Autowired
    AddrCategoryDao addrCategoryDao;

    @Autowired
    ManageTypeService manageTypeService;

    @Autowired
    TypeByDao byDao;


    @Bean("executor")
    public ThreadPoolTaskExecutor threadPoolTaskExecutor() {
        final ThreadPoolTaskExecutor threadPoolTaskExecutor = new ThreadPoolTaskExecutor();
        // 最佳线程数目 =((线程等待时间+线程CPU时间)/线程CPU时间 )*CPU数目
        // 核心线程池大小
        threadPoolTaskExecutor.setCorePoolSize(8);
        // 最大线程数
        threadPoolTaskExecutor.setMaxPoolSize(16);
        // 线程池维护线程(不包括核心线程)所允许的空闲时间, 秒
        threadPoolTaskExecutor.setKeepAliveSeconds(100);
        // 队列最大长度
        threadPoolTaskExecutor.setQueueCapacity(30);
        // 线程池关闭的时候等待所有任务都完成后, 再继续销毁其他的Bean
        threadPoolTaskExecutor.setWaitForTasksToCompleteOnShutdown(true);
        // 线程池中任务销毁的等待时间, 如果超过这个时间还没有销毁就强制销毁, 以确保应用最后能够被关闭, 而不是阻塞住
        threadPoolTaskExecutor.setAwaitTerminationSeconds(60);
        // 如果为true, 核心线程空闲时间到达setKeepAliveSeconds秒后会被销毁
        threadPoolTaskExecutor.setAllowCoreThreadTimeOut(false);
        // 线程前缀名称, 方便定位处理任务所在的线程池
        threadPoolTaskExecutor.setThreadNamePrefix("future-task-");
        /*
         拒绝策略
         采用默认策略, 线程池和队列都满了的时候, 丢弃任务 && 直接抛出java.util.concurrent.RejectedExecutionException异常
         */
        threadPoolTaskExecutor.setRejectedExecutionHandler(new ThreadPoolExecutor.AbortPolicy());

        // 初始化线程池
        threadPoolTaskExecutor.initialize();
        log.info("自定义线程池, ThreadPoolTaskExecutor线程池初始化 ... SUCCESS");

        return threadPoolTaskExecutor;
    }

    /**
     * HashMap<地区Code,城市等级>（redis版本）
     */
    @PostConstruct
    public void initCityTierMap() {
        if (RedisUtils.hasKey(RedisInitContant.INIT_CITY_TIER_MAP))
            return;
        log.info("正在加载CityTierMap");
        QueryWrapper<AddrCategory> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("type", "zone");
        queryWrapper.and(categoryQueryWrapper -> {
            categoryQueryWrapper.isNotNull("city_tier");
        });
        for (AddrCategory addrCategory : addrCategoryDao.selectList(queryWrapper)) {
            if (StrUtil.isNotEmpty(addrCategory.getCityTier())) {
                RedisUtils.setCacheMapValue(RedisInitContant.INIT_CITY_TIER_MAP, addrCategory.getAddrCode(), addrCategory.getCityTier());
            }
        }
        initCityTierListMap();
        log.info("CityTierMap已完成");
    }


    /**
     * 一线城市的addrCode（redis）
     */
    public void initCityTierListMap() {
        log.info("正在加载CityTierListMap");
        List<String> one = new ArrayList<>();
        List<String> two = new ArrayList<>();
        List<String> three = new ArrayList<>();
        List<String> four = new ArrayList<>();
        List<String> five = new ArrayList<>();
        List<String> newOne = new ArrayList<>();


        Set<String> keys = RedisUtils.getCacheMapKeySet(RedisInitContant.INIT_CITY_TIER_MAP);
        for (String s : keys) {
            String s1 = RedisUtils.getCacheMapValue(RedisInitContant.INIT_CITY_TIER_MAP, s);
            switch (s1) {
                case "一线":
                    one.add(s);
                    break;
                case "二线":
                    two.add(s);
                    break;
                case "三线":
                    three.add(s);
                    break;
                case "四线":
                    four.add(s);
                    break;
                case "五线":
                    five.add(s);
                    break;
                case "新一线":
                    newOne.add(s);
                    break;
            }
        }
        RedisUtils.setCacheMapValue(RedisInitContant.INIT_CITY_TIER_LIST_MAP, "一线", one);
        RedisUtils.setCacheMapValue(RedisInitContant.INIT_CITY_TIER_LIST_MAP, "二线", two);
        RedisUtils.setCacheMapValue(RedisInitContant.INIT_CITY_TIER_LIST_MAP, "三线", three);
        RedisUtils.setCacheMapValue(RedisInitContant.INIT_CITY_TIER_LIST_MAP, "四线", four);
        RedisUtils.setCacheMapValue(RedisInitContant.INIT_CITY_TIER_LIST_MAP, "五线", five);
        RedisUtils.setCacheMapValue(RedisInitContant.INIT_CITY_TIER_LIST_MAP, "新一线", newOne);
        log.info("CityTierListMap已完成");
    }

    @PostConstruct
    public void initManageType() {
        if (RedisUtils.hasKey(RedisInitContant.INIT_MANAGE_TYPE))
            return;
        log.info("正在加载ManageType");
        for (ManageType datum : manageTypeService.getData()) {
            if (!datum.getManageTypeCode().equals(""))
                RedisUtils.setCacheMapValue(RedisInitContant.INIT_MANAGE_TYPE, datum.getManageTypeCode(), datum);
        }
        log.info("ManageType已完成");
    }


    @PostConstruct
    public void initAddrCodeMap() {
        if (RedisUtils.hasKey(RedisInitContant.INIT_ADDR_CODE_MAP))
            return;
        log.info("正在加载AddrCodeMap");
        for (AddrCategory addrCode : addrCategoryDao.selectList(null)) {
            RedisUtils.setCacheMapValue(RedisInitContant.INIT_ADDR_CODE_MAP, addrCode.getAddrCode(), addrCode.getAddrInfo());
        }
        log.info("AddrCodeMap已完成");
    }


    @PostConstruct
    public void initTypeByMap() {
        if (RedisUtils.hasKey(RedisInitContant.INIT_TYPE_BY_MAP))
            return;
        log.info("正在加载TypeByMap");
        for (TypeBy datum : byDao.selectList(null)) {
            RedisUtils.setCacheMapValue(RedisInitContant.INIT_TYPE_BY_MAP, datum.getTypeCodeBy(), datum.getFullInfo());
        }
        log.info("TypeByMap已完成");
    }
}
