package com.ruoyi.demo.controller;

import cn.hutool.json.JSONObject;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.core.domain.R;
import com.ruoyi.demo.entity.Brand;
import com.ruoyi.demo.entity.bo.Histogram;
import com.ruoyi.demo.entity.bo.StoreWdCategoryCount;
import com.ruoyi.demo.entity.vo.ChannelAnalyseAceeptVo;
import com.ruoyi.demo.entity.vo.ChannelMapAceeptVo;
import com.ruoyi.demo.service.ChannelAnalyseService;
import com.ruoyi.demo.valida.ChannelAreaGroup;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.concurrent.ExecutionException;

/**
 * 渠道地图模块：渠道分析
 */
@RestController
@RequestMapping("/channelAnalyse")
public class ChannelAnalyseController {
    @Autowired
    ChannelAnalyseService channelAnalyseService;

    /**
     * 获取 网点城市等级分布统计
     *
     * @param channelMapAceeptVo 渠道地图过滤请求接收体
     * @return
     */
    @GetMapping("/cityTier")
    public R cityTier(@Validated(value = ChannelAreaGroup.class) ChannelMapAceeptVo channelMapAceeptVo) {
        HashMap<String, Long> hashMap = channelAnalyseService.cityTier(channelMapAceeptVo);
        return R.ok(hashMap);
    }

    /**
     * 获取 品牌列表
     *
     * @param channelAnalyseAceeptVo 渠道地图过滤请求接收体
     * @return
     */
    @GetMapping("/brandList")
    public R brandList(ChannelAnalyseAceeptVo channelAnalyseAceeptVo) {
        Page<Brand> brandPageInfo = channelAnalyseService.brandList(channelAnalyseAceeptVo);
        return R.ok(brandPageInfo);
    }

    /**
     * 获取 网点分类统计
     *
     * @param channelMapAceeptVo 渠道地图过滤请求接收体
     * @return
     */
    @GetMapping("/category")
    public R category(@Validated(value = ChannelAreaGroup.class) ChannelMapAceeptVo channelMapAceeptVo) {
        List<StoreWdCategoryCount> category = channelAnalyseService.category(channelMapAceeptVo);
        return R.ok(category);
    }

    /**
     * 网点周边标签分析
     *
     * @param channelMapAceeptVo
     * @return
     */
    @GetMapping("/tagAnalyse")
    public R tagAnalyse(@Validated(value = ChannelAreaGroup.class) ChannelMapAceeptVo channelMapAceeptVo) throws ExecutionException, InterruptedException {
        Integer page = channelMapAceeptVo.getPageNum() == null ? 1 : channelMapAceeptVo.getPageNum();
        channelMapAceeptVo.setPageNum(0);


        HashMap<String, Object> tagAnalyse = channelAnalyseService.tagAnalyse(channelMapAceeptVo);

        HashMap<String, Object> result = new HashMap<>();
        result.put("data", tagAnalyse.get(page + ""));
        result.put("pages", tagAnalyse.get("pages"));
        result.put("total", tagAnalyse.get("total"));
        return R.ok(result);
    }

    /**
     * 获取 网点营业状态统计
     *
     * @param channelMapAceeptVo 渠道地图过滤请求接收体
     * @return
     */

    @GetMapping("/businessStatusAnalyse")
    public R businessStatusAnalyse(@Validated(value = ChannelAreaGroup.class) ChannelMapAceeptVo channelMapAceeptVo) {

        JSONObject hashMap = channelAnalyseService.businessStatusAnalyse(channelMapAceeptVo);
        return R.ok(hashMap);
    }


    /**
     * 获取 楼宇类型数量统计
     *
     * @param channelMapAceeptVo 渠道地图过滤请求接收体
     * @return
     */

    @GetMapping("/aroundBuildAnalyse")
    public R aroundBuildAnalyse(@Validated(value = ChannelAreaGroup.class) ChannelMapAceeptVo channelMapAceeptVo) {

        ArrayList<Histogram> hashtable = channelAnalyseService.aroundBuildAnalyse(channelMapAceeptVo);
        return R.ok(hashtable);
    }

    /**
     * 获取 人均消费统计
     *
     * @param channelMapAceeptVo 渠道地图过滤请求接收体
     * @return
     */
    @GetMapping("/perCapitaConsumpAnalyse")
    public R perCapitaConsumpAnalyse(@Validated(value = ChannelAreaGroup.class) ChannelMapAceeptVo channelMapAceeptVo) {

        ArrayList<Histogram> hashtable = channelAnalyseService.perCapitaConsumpAnalyse(channelMapAceeptVo);
        return R.ok(hashtable);
    }

    /**
     * 获取 店龄/经营时长统计
     *
     * @param channelMapAceeptVo 渠道地图过滤请求接收体
     * @return
     */
    @GetMapping("/operateTimeAnalyse")
    public R operateTimeAnalyse(@Validated(value = ChannelAreaGroup.class) ChannelMapAceeptVo channelMapAceeptVo) {
        ArrayList<Histogram> hashtable = channelAnalyseService.operateTimeAnalyse(channelMapAceeptVo);
        return R.ok(hashtable);
    }


}
