package com.ruoyi.demo.controller;


import com.ruoyi.common.core.domain.R;
import com.ruoyi.demo.entity.bo.HeatMapBo;
import com.ruoyi.demo.entity.bo.Histogram;
import com.ruoyi.demo.entity.bo.StoreWdCategoryCount;
import com.ruoyi.demo.entity.vo.TopologicalWdAceeptVo;
import com.ruoyi.demo.service.TopologicalStoreWdService;
import com.ruoyi.demo.valida.TopologicalWdGroup;
import com.ruoyi.demo.valida.TopologicalWdListGroup;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.List;
import java.util.concurrent.ExecutionException;

/**
 * 点位评估模块：
 */
@RestController
@RequestMapping("/poi/topologicalStoreWd")
public class TopologicalStoreWdController {

    @Autowired
    TopologicalStoreWdService topologicalStoreWdService;


    /**
     * 获取周边网点分类
     *
     * @param topologicalWdAceeptVo 周边网点接受体
     * @return
     */
    @GetMapping("/category")
    public R category(@Validated(value = {TopologicalWdGroup.class}) TopologicalWdAceeptVo topologicalWdAceeptVo) throws ExecutionException, InterruptedException {

        List<StoreWdCategoryCount> category = topologicalStoreWdService.category(topologicalWdAceeptVo);
        return R.ok(category);
    }

    /**
     * 获取周边门店网点列表
     *
     * @param topologicalWdAceeptVo 周边网点接受体
     * @return
     */
    @GetMapping("/list")
    public R list(@Validated(value = {TopologicalWdListGroup.class}) TopologicalWdAceeptVo topologicalWdAceeptVo) throws ExecutionException, InterruptedException {
        HashMap<String, Object> list = topologicalStoreWdService.list(topologicalWdAceeptVo);
        return R.ok(list);
    }

    /**
     * 获取商业网点分类数量统计
     *
     * @param topologicalWdAceeptVo 周边网点接受体
     * @return
     */
    @GetMapping("/storeHistogram")
    public R storeHistogram(@Validated(value = {TopologicalWdGroup.class}) TopologicalWdAceeptVo topologicalWdAceeptVo) throws ExecutionException, InterruptedException {
        List<Histogram> list = topologicalStoreWdService.storeHistogram(topologicalWdAceeptVo);
        return R.ok(list);
    }

    /**
     * 获取门店人均消费
     *
     * @param topologicalWdAceeptVo 周边网点接受体
     * @return
     */
    @GetMapping("/perCapitaConsumpHistogram")
    public R perCapitaConsumpHistogram(@Validated(value = {TopologicalWdGroup.class}) TopologicalWdAceeptVo topologicalWdAceeptVo) throws ExecutionException, InterruptedException {
        List<Histogram> list = topologicalStoreWdService.perCapitaConsumpHistogram(topologicalWdAceeptVo);
        return R.ok(list);
    }

    /**
     * 获取门店人均消费
     *
     * @param topologicalWdAceeptVo 周边网点接受体
     * @return
     */
    @GetMapping("/heatMap")
    public R heatMap(@Validated(value = {TopologicalWdGroup.class}) TopologicalWdAceeptVo topologicalWdAceeptVo) throws ExecutionException, InterruptedException {
        List<HeatMapBo> heatMapBos = topologicalStoreWdService.heatMap(topologicalWdAceeptVo);
        return R.ok(heatMapBos);
    }
}
