package com.ruoyi.demo.service.impl;

import cn.hutool.core.map.MapUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.utils.redis.RedisUtils;
import com.ruoyi.demo.constant.RedisContant;
import com.ruoyi.demo.entity.WdInfo;
import com.ruoyi.demo.entity.bo.PointBo;
import com.ruoyi.demo.entity.vo.TopologicalWdAceeptVo;
import com.ruoyi.demo.mapper.WdInfoDao;
import com.ruoyi.demo.service.TopologicalCommonService;
import com.ruoyi.demo.utils.InitMapUtil;
import org.redisson.api.GeoUnit;
import org.redisson.api.RFuture;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.concurrent.ExecutionException;
import java.util.stream.Collectors;

@Service
@Transactional
public class TopologicalCommonServiceImpl implements TopologicalCommonService {
    @Autowired
    WdInfoDao wdInfoDao;

    @Autowired
    InitMapUtil initMapUtil;


    /**
     * 点位评估中心点查询
     *
     * @param searchText 收索关键字
     * @param size       返回条数
     * @return
     */
    @Override
    public List<WdInfo> prefixSearch(String searchText, Integer size) {
        QueryWrapper<WdInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.likeRight("wd_name", searchText);
        queryWrapper.and(wdInfoQueryWrapper -> {
            wdInfoQueryWrapper.in("wd_type_code", Arrays.asList("1", "2", "3", "5"));
        });

        int pageSize = size == null ? 8 : size;
        Page<WdInfo> page = new Page<>(1, pageSize);
        Page<WdInfo> page1 = wdInfoDao.selectPage(page, queryWrapper);

        return page1.getRecords();
    }

    /**
     * 获取周边点坐标
     *
     * @param topologicalWdAceeptVo 周边网点接受体
     * @return
     */
    @Cacheable(cacheNames = RedisContant.TOPOLOGICAL_COMMON_MAP + "#8h", key = "#topologicalWdAceeptVo.getHash()", unless = "#result == null")
    @Override
    public List<PointBo> map(TopologicalWdAceeptVo topologicalWdAceeptVo) throws ExecutionException, InterruptedException {
        List<String> aroundWdId = new ArrayList<>();

        //1.根据经纬度 找到 周边网点Id 及 距离
        getAroundWdIdList(topologicalWdAceeptVo, aroundWdId, null);
        if (aroundWdId.isEmpty())
            return new ArrayList<>();

        //2.查询符合的周边网点信息
        QueryWrapper<WdInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.select("wd_id", "wd_name", "lat", "lng", "addr_code")
            .in("wd_id", aroundWdId);

        return wdInfoDao.map(queryWrapper);
    }

    /**
     * 根据坐标获取周边网点Id
     *
     * @param topologicalWdAceeptVo 周边网点接受体
     * @param aroundWdIdList        存放结果存放的 网点Id集合
     * @param wdDistMap             存放结果的 周边网点与中心网点直接的距离
     */
    public void getAroundWdIdList(TopologicalWdAceeptVo topologicalWdAceeptVo, List<String> aroundWdIdList, HashMap<String, Double> wdDistMap) throws ExecutionException, InterruptedException {

        RFuture<Map<Object, Double>> mapRFuture = RedisUtils.getClient().getGeo(topologicalWdAceeptVo.getAddrCode().substring(0, 4)).radiusWithDistanceAsync(topologicalWdAceeptVo.getLng(), topologicalWdAceeptVo.getLat(), topologicalWdAceeptVo.getRadius(), GeoUnit.METERS);
        Map<Object, Double> map = mapRFuture.get();
        if (MapUtil.isNotEmpty(map)) {
            Map<String, Double> distMap = map.entrySet().stream().collect(Collectors.toMap(entry -> entry.getKey().toString(), Map.Entry::getValue));
            aroundWdIdList.addAll(distMap.keySet());
            if (wdDistMap != null) wdDistMap.putAll(distMap);
        }
    }

}
