package com.ruoyi.demo.service.impl;

import cn.hutool.core.map.MapUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.utils.redis.RedisUtils;
import com.ruoyi.demo.constant.RedisContant;
import com.ruoyi.demo.entity.TopologicalEnterpriseWd;
import com.ruoyi.demo.entity.WdInfo;
import com.ruoyi.demo.entity.vo.TopologicalWdAceeptVo;
import com.ruoyi.demo.mapper.WdInfoDao;
import com.ruoyi.demo.service.TopologicalEnterpriseWdService;
import com.ruoyi.demo.utils.InitMapUtil;
import org.redisson.api.GeoUnit;
import org.redisson.api.RFuture;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ExecutionException;
import java.util.stream.Collectors;

@Service
@Transactional
public class TopologicalEnterpriseWdServiceImpl implements TopologicalEnterpriseWdService {

    @Autowired
    WdInfoDao wdInfoDao;

    @Autowired
    InitMapUtil initMapUtil;


    /**
     * 获取周边企业网点列表
     *
     * @param topologicalWdAceeptVo 周边网点接受体
     * @return
     */
    @Cacheable(cacheNames = RedisContant.TOPOLOGICAL_ENTERPRISE_WD_LIST + "#8h", key = "#topologicalWdAceeptVo.getHash()", unless = "#result == null")
    @Override
    public HashMap<String, Object> list(TopologicalWdAceeptVo topologicalWdAceeptVo) throws ExecutionException, InterruptedException {
        List<String> aroundWdId = new ArrayList<>();
        HashMap<String, Double> wdDistMap = new HashMap<>();

        //1.根据经纬度 找到 周边网点Id 及 距离
        getAroundWdIdList(topologicalWdAceeptVo, aroundWdId, wdDistMap);
        if (aroundWdId.isEmpty()) {
            //没有周边网点
            HashMap<String, Object> result = new HashMap();
            Page<WdInfo> page = new Page<>(topologicalWdAceeptVo.getPageNum(), topologicalWdAceeptVo.getPageSize());
            result.put("page", page);
            result.put("data", new ArrayList<>());
            return result;
        }

        //2.分页获取周边企业网点信息
        QueryWrapper<WdInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("wd_type_code", "5");
        queryWrapper.and(wdInfoQueryWrapper -> {
            wdInfoQueryWrapper.in("wd_id", aroundWdId);
        });
        Page<WdInfo> page = new Page<>(topologicalWdAceeptVo.getPageNum(), topologicalWdAceeptVo.getPageSize());
        Page<WdInfo> page1 = wdInfoDao.selectPage(page, queryWrapper);


        //3.组装结果集
        List<TopologicalEnterpriseWd> list = new ArrayList<>();  //返回结果集
        for (WdInfo wdInfo : page1.getRecords()) {
            TopologicalEnterpriseWd topologicalEnterpriseWd = new TopologicalEnterpriseWd(wdInfo);
            //topologicalEnterpriseWd.setAddrCodeInfo(initMapUtil.getInitAddrCodeMap(topologicalEnterpriseWd.getAddrCode()));
            BigDecimal bigDecimal = new BigDecimal(wdDistMap.get(wdInfo.getWdId()));
            topologicalEnterpriseWd.setDistance(bigDecimal.setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue());
            list.add(topologicalEnterpriseWd);
        }

        //4.返回结果
        HashMap<String, Object> result = new HashMap();
        page1.setRecords(null);
        result.put("page", page1);
        result.put("data", list);
        return result;
    }

    /**
     * 获取周边物流园
     *
     * @param topologicalWdAceeptVo 周边网点接受体
     * @return
     */
    @Cacheable(cacheNames = RedisContant.TOPOLOGICAL_ENTERPRISE_WD_LOGISTICS_LIST + "#8h", key = "#topologicalWdAceeptVo.getHash()", unless = "#result == null")
    @Override
    public HashMap<String, Object> logisticsList(TopologicalWdAceeptVo topologicalWdAceeptVo) throws ExecutionException, InterruptedException {
        List<String> aroundWdId = new ArrayList<>();
        HashMap<String, Double> wdDistMap = new HashMap<>();

        //1.根据经纬度 找到 周边网点Id 及 距离
        getAroundWdIdList(topologicalWdAceeptVo, aroundWdId, wdDistMap);
        if (aroundWdId.isEmpty()) {
            //没有周边网点
            HashMap<String, Object> result = new HashMap();
            Page<WdInfo> page = new Page<>(topologicalWdAceeptVo.getPageNum(), topologicalWdAceeptVo.getPageSize());
            result.put("page", page);
            result.put("data", new ArrayList<>());
            return result;
        }

        //2.分页获取周边企业物流园网点信息
        QueryWrapper<WdInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("wd_type_code", "5");
        queryWrapper.and(wdInfoQueryWrapper -> {
            wdInfoQueryWrapper.in("wd_id", aroundWdId);
        });
        queryWrapper.and(wdInfoQueryWrapper -> {
            wdInfoQueryWrapper.like("wd_name", "物流");
        });
        Page<WdInfo> page = new Page<>(topologicalWdAceeptVo.getPageNum(), topologicalWdAceeptVo.getPageSize());
        Page<WdInfo> page1 = wdInfoDao.selectPage(page, queryWrapper);


        //3.组装结果集
        List<TopologicalEnterpriseWd> list = new ArrayList<>();  //返回结果集
        for (WdInfo wdInfo : page1.getRecords()) {
            TopologicalEnterpriseWd topologicalEnterpriseWd = new TopologicalEnterpriseWd(wdInfo);
            //topologicalEnterpriseWd.setAddrCodeInfo(initMapUtil.getInitAddrCodeMap(topologicalEnterpriseWd.getAddrCode()));
            BigDecimal bigDecimal = new BigDecimal(wdDistMap.get(wdInfo.getWdId()));
            topologicalEnterpriseWd.setDistance(bigDecimal.setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue());
            list.add(topologicalEnterpriseWd);
        }

        //4.返回结果
        HashMap<String, Object> result = new HashMap<>();
        page1.setRecords(null);
        result.put("page", page1);
        result.put("data", list);
        return result;
    }

    /**
     * 根据坐标获取周边网点Id
     *
     * @param topologicalWdAceeptVo 周边网点接受体
     * @param aroundWdIdList        存放结果存放的 网点Id集合
     * @param wdDistMap             存放结果的 周边网点与中心网点直接的距离
     */
    public void getAroundWdIdList(TopologicalWdAceeptVo topologicalWdAceeptVo, List<String> aroundWdIdList, HashMap<String, Double> wdDistMap) throws ExecutionException, InterruptedException {
        RFuture<Map<Object, Double>> mapRFuture = RedisUtils.getClient().getGeo(topologicalWdAceeptVo.getAddrCode().substring(0, 4)).radiusWithDistanceAsync(topologicalWdAceeptVo.getLng(), topologicalWdAceeptVo.getLat(), topologicalWdAceeptVo.getRadius(), GeoUnit.METERS);
        Map<Object, Double> map = mapRFuture.get();
        if (MapUtil.isNotEmpty(map)) {
            Map<String, Double> distMap = map.entrySet().stream().collect(Collectors.toMap(entry -> entry.getKey().toString(), Map.Entry::getValue));
            aroundWdIdList.addAll(distMap.keySet());
            if (wdDistMap != null) wdDistMap.putAll(distMap);
        }
    }
}
